# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object :py:class:`wwdtm.show.ShowUtility`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.show import ShowUtility


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("year, month, day", [(2018, 10, 27)])
def test_show_utility_convert_date_to_id(year: int, month: int, day: int):
    """Testing for :py:meth:`wwdtm.show.ShowUtility.convert_date_to_id`.

    :param year: Four digit year to test converting into show ID
    :param month: One or two digit month to test converting into show ID
    :param day: One or two digit day to test converting into show ID
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    id_ = utility.convert_date_to_id(year, month, day)

    assert id_, f"Show ID for date {year:04d}-{month:02d}-{day:02d} not found"
    assert isinstance(
        id_, int
    ), f"Invalid value returned for date {year:04d}-{month:02d}-{day:02d}"


@pytest.mark.parametrize("year, month, day", [(2018, 10, 26)])
def test_show_utility_convert_invalid_date_to_id(year: int, month: int, day: int):
    """Negative testing for :py:meth:`wwdtm.show.ShowUtility.convert_date_to_id`.

    :param year: Four digit year to test failing to convert into show ID
    :param month: One or two digit month to test failing to convert
        into show ID
    :param day: One or two digit day to test failing to convert into
        show ID
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    id_ = utility.convert_date_to_id(year, month, day)

    assert not id_, f"Show ID for date {year:04d}-{month:02d}-{day:02d} was found"


@pytest.mark.parametrize("show_id", [1162])
def test_show_utility_convert_id_to_date(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowUtility.convert_id_to_date`.

    :param show_id: Show ID to test converting into show date
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    date = utility.convert_id_to_date(show_id)

    assert date, f"Show date for ID {show_id} was not found"
    assert isinstance(date, str), f"Invalid value returned for ID {show_id}"


@pytest.mark.parametrize("show_id", [-1])
def test_show_utility_convert_invalid_id_to_date(show_id: int):
    """Negative testing for :py:meth:`wwdtm.show.ShowUtility.convert_id_to_date`.

    :param show_id: Show ID to test failing to convert into show date
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    date = utility.convert_id_to_date(show_id)

    assert not date, f"Show date for ID {show_id} was found"


@pytest.mark.parametrize("year, month, day", [(2020, 4, 25)])
def test_show_utility_date_exists(year: int, month: int, day: int):
    """Testing for :py:meth:`wwdtm.show.ShowUtility.date_exists`.

    :param year: Four digit year to test if a show exists
    :param month: One or two digit month to test if a show exists
    :param day: One or two digit day to test if a show exists
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    result = utility.date_exists(year, month, day)

    assert result, f"Show date {year:04d}-{month:02d}-{day:02d} was not found"


@pytest.mark.parametrize("year, month, day", [(2020, 4, 24)])
def test_show_utility_date_not_exists(year: int, month: int, day: int):
    """Negative testing for :py:meth:`wwdtm.show.ShowUtility.date_exists`.

    :param year: Four digit year to test if a show does not exist
    :param month: One or two digit month to test if a show does not
        exist
    :param day: One or two digit day to test if a show does not exist
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    result = utility.date_exists(year, month, day)

    assert not result, f"Show date {year:04d}-{month:02d}-{day:02d} was found"


@pytest.mark.parametrize("show_id", [1162])
def test_show_utility_id_exists(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowUtility.id_exists`.

    :param show_id: Show ID to test if a show exists
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(show_id)

    assert result, f"Show ID {show_id} was not found"


@pytest.mark.parametrize("show_id", [-1])
def test_show_utility_id_not_exists(show_id: int):
    """Negative testing for :py:meth:`wwdtm.show.ShowUtility.id_exists`.

    :param show_id: Show ID to test if a show does not exist
    """
    utility = ShowUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(show_id)

    assert not result, f"Show ID {show_id} was found"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.host.HostAppearances`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.host import HostAppearances


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("host_id", [2])
def test_host_appearances_retrieve_appearances_by_id(host_id: int):
    """Testing for :py:meth:`wwdtm.host.HostAppearances.retrieve_appearances_by_id`.

    :param host_id: Host ID to test retrieving host appearances
    """
    appearances = HostAppearances(connect_dict=get_connect_dict())
    appearance = appearances.retrieve_appearances_by_id(host_id)

    assert "count" in appearance, f"'count' was not returned for ID {host_id}"
    assert "shows" in appearance, f"'shows' was not returned for ID {host_id}"


@pytest.mark.parametrize("host_slug", ["luke-burbank"])
def test_host_appearances_retrieve_appearances_by_slug(host_slug: str):
    """Testing for :py:meth:`wwdtm.host.HostAppearances.retrieve_appearances_by_slug`.

    :param host_slug: Host slug string to test retrieving host
        appearances
    """
    appearances = HostAppearances(connect_dict=get_connect_dict())
    appearance = appearances.retrieve_appearances_by_slug(host_slug)

    assert "count" in appearance, f"'count' was not returned for slug {host_slug}"
    assert "shows" in appearance, f"'shows' was not returned for slug {host_slug}"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Show Detailed Information Retrieval Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.location.location import LocationUtility
from wwdtm.show.utility import ShowUtility
from wwdtm.validation import valid_int_id


class ShowInfo:
    """Show information retrieval class.

    Contains methods used to retrieve panelist, guest and Bluff the
    Listener information.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = ShowUtility(database_connection=self.database_connection)
        self.loc_util = LocationUtility(database_connection=self.database_connection)
        self.panelists = self._retrieve_panelists()

    def _retrieve_panelists(self) -> dict[int, dict[str, str]]:
        """Returns a dictionary of panelist information.

        :return: Dictionary containing panelist ID as the key and
            panelist name and slug string as a dictionary as a value.
        """
        query = """
            SELECT panelistid, panelist, panelistslug
            FROM ww_panelists
            ORDER BY panelistid ASC;
        """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        panelists = {}
        for row in results:
            panelists[row.panelistid] = {
                "name": row.panelist,
                "slug": row.panelistslug,
            }

        return panelists

    def retrieve_bluff_info_by_id(self, show_id: int) -> list[dict[str, Any]]:
        """Retrieves Bluff the Listener information.

        :param show_id: Show ID
        :return: A dictionary containing Bluff the Listener segment ID
            and information about the chosen Bluff panelist and correct
            Bluff panelist.
        """
        if not valid_int_id(show_id):
            return {}

        query = """
            SELECT segment, chosenbluffpnlid AS chosen_id,
            correctbluffpnlid AS correct_id
            FROM ww_showbluffmap
            WHERE showid = %s
            ORDER BY segment ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (show_id,))
        result = cursor.fetchall()

        if not result:
            return []

        bluffs = []
        for row in result:
            if not row.chosen_id and not row.correct_id:
                bluffs.append(
                    {
                        "segment": row.segment,
                        "chosen_panelist": None,
                        "correct_panelist": None,
                    }
                )
            elif row.chosen_id and not row.correct_id:
                bluffs.append(
                    {
                        "segment": row.segment,
                        "chosen_panelist": {
                            "id": row.chosen_id,
                            "name": self.panelists[row.chosen_id]["name"],
                            "slug": self.panelists[row.chosen_id]["slug"]
                            if self.panelists[row.chosen_id]["slug"]
                            else slugify(self.panelists[row.chosen_id]["name"]),
                        },
                        "correct_panelist": None,
                    }
                )
            elif row.correct_id and not row.chosen_id:
                bluffs.append(
                    {
                        "segment": row.segment,
                        "chosen_panelist": None,
                        "correct_panelist": {
                            "id": row.correct_id,
                            "name": self.panelists[row.correct_id]["name"],
                            "slug": self.panelists[row.correct_id]["slug"]
                            if self.panelists[row.correct_id]["slug"]
                            else slugify(self.panelists[row.correct_id]["name"]),
                        },
                    }
                )
            else:
                bluffs.append(
                    {
                        "segment": row.segment,
                        "chosen_panelist": {
                            "id": row.chosen_id,
                            "name": self.panelists[row.chosen_id]["name"],
                            "slug": self.panelists[row.chosen_id]["slug"]
                            if self.panelists[row.chosen_id]["slug"]
                            else slugify(self.panelists[row.chosen_id]["name"]),
                        },
                        "correct_panelist": {
                            "id": row.correct_id,
                            "name": self.panelists[row.correct_id]["name"],
                            "slug": self.panelists[row.correct_id]["slug"]
                            if self.panelists[row.correct_id]["slug"]
                            else slugify(self.panelists[row.correct_id]["name"]),
                        },
                    }
                )

        return bluffs

    def retrieve_core_info_by_id(self, show_id: int) -> dict[str, Any]:
        """Retrieves core information.

        :param show_id: Show ID
        :return: A dictionary containing host, scorekeeper, location,
            description and notes
        """
        if not valid_int_id(show_id):
            return {}

        query = """
            SELECT s.showid AS show_id, s.showdate AS date,
            s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
            s.showurl AS show_url,
            l.locationid AS location_id, l.city, l.state,
            l.venue, l.locationslug AS location_slug, h.hostid AS host_id,
            h.host, h.hostslug AS host_slug, hm.guest as host_guest,
            sk.scorekeeperid AS scorekeeper_id, sk.scorekeeper,
            sk.scorekeeperslug AS scorekeeper_slug,
            skm.guest AS scorekeeper_guest,
            skm.description AS scorekeeper_description,
            sd.showdescription AS show_description,
            sn.shownotes AS show_notes
            FROM ww_shows s
            JOIN ww_showlocationmap lm ON lm.showid = s.showid
            JOIN ww_locations l ON l.locationid = lm.locationid
            JOIN ww_showhostmap hm ON hm.showid = s.showid
            JOIN ww_hosts h ON h.hostid = hm.hostid
            JOIN ww_showskmap skm ON skm.showid = s.showid
            JOIN ww_scorekeepers sk ON sk.scorekeeperid = skm.scorekeeperid
            JOIN ww_showdescriptions sd ON sd.showid = s.showid
            JOIN ww_shownotes sn ON sn.showid = s.showid
            WHERE s.showid = %s
            ORDER BY s.showdate ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (show_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        location_info = {
            "id": result.location_id,
            "slug": result.location_slug,
            "city": result.city,
            "state": result.state,
            "venue": result.venue,
        }

        if not result.location_slug:
            location_info["slug"] = self.loc_util.slugify_location(
                location_id=result.location_id,
                venue=result.venue,
                city=result.city,
                state=result.state,
            )

        host_info = {
            "id": result.host_id,
            "name": result.host,
            "slug": result.host_slug if result.host_slug else slugify(result.host),
            "guest": bool(result.host_guest),
        }

        scorekeeper_info = {
            "id": result.scorekeeper_id,
            "name": result.scorekeeper,
            "slug": result.scorekeeper_slug
            if result.scorekeeper_slug
            else slugify(result.scorekeeper),
            "guest": bool(result.scorekeeper_guest),
            "description": result.scorekeeper_description
            if result.scorekeeper_description
            else None,
        }

        if result.show_description:
            description = str(result.show_description).strip()
        else:
            description = None

        notes = str(result.show_notes).strip() if result.show_notes else None

        show_info = {
            "id": result.show_id,
            "date": result.date.isoformat(),
            "best_of": bool(result.best_of),
            "repeat_show": bool(result.repeat_show_id),
            "original_show_id": None,
            "original_show_date": None,
            "show_url": result.show_url,
            "description": description,
            "notes": notes,
            "location": location_info,
            "host": host_info,
            "scorekeeper": scorekeeper_info,
        }

        repeat_show_id = result.repeat_show_id
        if repeat_show_id:
            original_date = self.utility.convert_id_to_date(repeat_show_id)
            show_info["original_show_id"] = repeat_show_id
            show_info["original_show_date"] = original_date
        else:
            show_info.pop("original_show_id", None)
            show_info.pop("original_show_date", None)

        return show_info

    def retrieve_guest_info_by_id(self, show_id: int) -> list[dict[str, Any]]:
        """Retrieves Not My Job guest information.

        :param show_id: Show ID
        :return: A dictionary containing Not My Job guest information,
            including score and scoring exception for each guest
        """
        if not valid_int_id(show_id):
            return []

        query = """
            SELECT gm.guestid AS id, g.guest AS name,
            g.guestslug AS slug, gm.guestscore AS score,
            gm.exception AS score_exception
            FROM ww_showguestmap gm
            JOIN ww_guests g on g.guestid = gm.guestid
            JOIN ww_shows s on s.showid = gm.showid
            WHERE gm.showid = %s
            ORDER by gm.showguestmapid ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (show_id,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        guests = []
        for guest in results:
            guests.append(
                {
                    "id": guest.id,
                    "name": guest.name,
                    "slug": guest.slug if guest.slug else slugify(guest.name),
                    "score": guest.score,
                    "score_exception": bool(guest.score_exception),
                }
            )

        return guests

    def retrieve_panelist_info_by_id(
        self, show_id: int, include_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves panelist information.

        :param show_id: Show ID
        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A dictionary containing panelist information, scores
            and rankings
        """
        if not valid_int_id(show_id):
            return []

        if include_decimal_scores:
            query = """
                SELECT pm.panelistid AS id, p.panelist AS name,
                p.panelistslug AS slug,
                pm.panelistlrndstart AS start,
                pm.panelistlrndstart_decimal AS start_decimal,
                pm.panelistlrndcorrect AS correct,
                pm.panelistlrndcorrect_decimal AS correct_decimal,
                pm.panelistscore AS score,
                pm.panelistscore_decimal AS score_decimal,
                pm.showpnlrank AS pnl_rank
                FROM ww_showpnlmap pm
                JOIN ww_panelists p on p.panelistid = pm.panelistid
                WHERE pm.showid = %s
                ORDER by pm.panelistscore_decimal DESC,
                pm.showpnlmapid ASC;
                """
        else:
            query = """
                SELECT pm.panelistid AS id, p.panelist AS name,
                p.panelistslug AS slug,
                pm.panelistlrndstart AS start,
                pm.panelistlrndcorrect AS correct,
                pm.panelistscore AS score, pm.showpnlrank AS pnl_rank
                FROM ww_showpnlmap pm
                JOIN ww_panelists p on p.panelistid = pm.panelistid
                WHERE pm.showid = %s
                ORDER by pm.panelistscore DESC, pm.showpnlmapid ASC;
                """

        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (show_id,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        panelists = []
        for row in results:
            panelists.append(
                {
                    "id": row.id,
                    "name": row.name,
                    "slug": row.slug if row.slug else slugify(row.name),
                    "lightning_round_start": row.start,
                    "lightning_round_start_decimal": row.start_decimal
                    if "start_decimal" in row._fields
                    else None,
                    "lightning_round_correct": row.correct,
                    "lightning_round_correct_decimal": row.correct_decimal
                    if "correct_decimal" in row._fields
                    else None,
                    "score": row.score,
                    "score_decimal": row.score_decimal
                    if "score_decimal" in row._fields
                    else None,
                    "rank": row.pnl_rank if row.pnl_rank else None,
                }
            )

        return panelists



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.panelist.PanelistUtility`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.panelist import PanelistUtility


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("panelist_id", [14])
def test_panelist_utility_convert_id_to_slug(panelist_id: int):
    """Testing for :py:meth:`wwdtm.panelist.PanelistUtility.convert_id_to_slug`.

    :param panelist_id: Panelist ID to test converting into panelist
        slug string
    """
    utility = PanelistUtility(get_connect_dict())
    slug = utility.convert_id_to_slug(panelist_id)

    assert slug, f"Panelist slug for ID {panelist_id} was not found"
    assert isinstance(slug, str), f"Invalid value returned for ID {panelist_id}"


@pytest.mark.parametrize("panelist_id", [-1])
def test_panelist_utility_convert_invalid_id_to_slug(panelist_id: int):
    """Negative testing for :py:meth:`wwdtm.panelist.PanelistUtility.convert_id_to_slug`.

    :param panelist_id: Panelist ID to test failing to convert into
        panelist slug string
    """
    utility = PanelistUtility(get_connect_dict())
    slug = utility.convert_id_to_slug(panelist_id)

    assert not slug, f"Panelist slug for ID {panelist_id} was found"


@pytest.mark.parametrize("panelist_slug", ["faith-salie"])
def test_panelist_utility_convert_slug_to_id(panelist_slug: str):
    """Testing for :py:meth:`wwdtm.panelist.PanelistUtility.convert_slug_to_id`.

    :param panelist_slug: Panelist slug string to test converting into
        panelist ID
    """
    utility = PanelistUtility(get_connect_dict())
    id_ = utility.convert_slug_to_id(panelist_slug)

    assert id_, f"Panelist ID for slug {panelist_slug} was not found"
    assert isinstance(id_, int), f"Invalid value returned for slug {panelist_slug}"


@pytest.mark.parametrize("panelist_slug", ["faith-sale"])
def test_panelist_utility_convert_invalid_slug_to_id(panelist_slug: str):
    """Negative testing for :py:meth:`wwdtm.panelist.PanelistUtility.convert_slug_to_id`.

    :param panelist_slug: Panelist slug string to test failing to
        convert into panelist ID
    """
    utility = PanelistUtility(get_connect_dict())
    id_ = utility.convert_slug_to_id(panelist_slug)

    assert not id_, f"Panelist ID for slug {panelist_slug} was found"


@pytest.mark.parametrize("panelist_id", [14])
def test_panelist_utility_id_exists(panelist_id: int):
    """Testing for :py:meth:`wwdtm.panelist.PanelistUtility.id_exists`.

    :param panelist_id: Panelist ID to test if a panelist exists
    """
    utility = PanelistUtility(get_connect_dict())
    result = utility.id_exists(panelist_id)

    assert result, f"Panelist ID {panelist_id} does not exist"


@pytest.mark.parametrize("panelist_id", [-1])
def test_panelist_utility_id_not_exists(panelist_id: int):
    """Negative testing for :py:meth:`wwdtm.panelist.PanelistUtility.id_exists`.

    :param panelist_id: Panelist ID to test if a panelist does not exist
    """
    utility = PanelistUtility(get_connect_dict())
    result = utility.id_exists(panelist_id)

    assert not result, f"Panelist ID {panelist_id} exists"


@pytest.mark.parametrize("panelist_slug", ["faith-salie"])
def test_panelist_utility_slug_exists(panelist_slug: str):
    """Testing for :py:meth:`wwdtm.panelist.PanelistUtility.slug_exists`.

    :param panelist_slug: Panelist slug string to test if a panelist
        exists
    """
    utility = PanelistUtility(get_connect_dict())
    result = utility.slug_exists(panelist_slug)

    assert result, f"Panelist slug {panelist_slug} does not exist"


@pytest.mark.parametrize("panelist_slug", ["faith-sale"])
def test_panelist_utility_slug_not_exists(panelist_slug: str):
    """Negative testing for :py:meth:`wwdtm.panelist.PanelistUtility.slug_exists`.

    :param panelist_slug: Panelist slug string to test if a panelist
        does not exist
    """
    utility = PanelistUtility(get_connect_dict())
    result = utility.slug_exists(panelist_slug)

    assert not result, f"Panelist slug {panelist_slug} exists"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Location Data Retrieval Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.location.recordings import LocationRecordings
from wwdtm.location.utility import LocationUtility
from wwdtm.validation import valid_int_id


class Location:
    """Location information retrieval class.

    Contains methods used to retrieve location IDs, venue names and
    locations, including city and state, and recording dates.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.recordings = LocationRecordings(
            database_connection=self.database_connection
        )
        self.utility = LocationUtility(database_connection=self.database_connection)

    def retrieve_all(self, sort_by_venue: bool = False) -> list[dict[str, int | str]]:
        """Retrieves location information for all locations.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of dictionaries containing location ID, venue
            name, city, state and slug string
        """
        query = """
            SELECT locationid AS id, city, state, venue,
            locationslug AS slug
            FROM ww_locations
            """

        if sort_by_venue:
            query = query + "ORDER BY venue ASC, city ASC, state ASC;"
        else:
            query = query + "ORDER BY state ASC, city ASC, venue ASC;"

        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        locations = []
        for row in results:
            locations.append(
                {
                    "id": row.id,
                    "city": row.city,
                    "state": row.state,
                    "venue": row.venue,
                    "slug": row.slug
                    if row.slug
                    else self.utility.slugify_location(
                        location_id=row.id,
                        venue=row.venue,
                        city=row.city,
                        state=row.state,
                    ),
                }
            )

        return locations

    def retrieve_all_details(self, sort_by_venue: bool = False) -> list[dict[str, Any]]:
        """Retrieves location information and recordings for all locations.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of dictionaries containing location ID, venue
            name, city, state, slug string and a list of recordings
        """
        query = """
            SELECT locationid AS id, city, state, venue,
            locationslug AS slug
            FROM ww_locations
            """
        if sort_by_venue:
            query = query + "ORDER BY venue ASC, city ASC, state ASC;"
        else:
            query = query + "ORDER BY state ASC, city ASC, venue ASC;"

        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        locations = []
        for row in results:
            locations.append(
                {
                    "id": row.id,
                    "city": row.city,
                    "state": row.state,
                    "venue": row.venue,
                    "slug": row.slug
                    if row.slug
                    else self.utility.slugify_location(
                        location_id=row.id,
                        venue=row.venue,
                        city=row.city,
                        state=row.state,
                    ),
                    "recordings": self.recordings.retrieve_recordings_by_id(row.id),
                }
            )

        return locations

    def retrieve_all_ids(self, sort_by_venue: bool = False) -> list[int]:
        """Retrieves all location IDs.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of location IDs as integers
        """
        query = "SELECT locationid FROM ww_locations "
        if sort_by_venue:
            query = query + "ORDER BY venue ASC, city ASC, state ASC;"
        else:
            query = query + "ORDER BY state ASC, city ASC, venue ASC;"

        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self, sort_by_venue: bool = False) -> list[str]:
        """Retrieves all location slug strings.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of location slug strings
        """
        query = "SELECT locationslug FROM ww_locations "
        if sort_by_venue:
            query = query + "ORDER BY venue ASC, city ASC, state ASC;"
        else:
            query = query + "ORDER BY state ASC, city ASC, venue ASC;"

        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, location_id: int) -> dict[str, int | str]:
        """Retrieves location information.

        :param location_id: Location ID
        :return: A dictionary containing location ID, venue, city, state
            and slug string
        """
        if not valid_int_id(location_id):
            return {}

        query = """
            SELECT locationid AS id, city, state, venue,
            locationslug AS slug
            FROM ww_locations
            WHERE locationid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (location_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "id": result.id,
            "city": result.city,
            "state": result.state,
            "venue": result.venue,
            "slug": result.slug
            if result.slug
            else self.utility.slugify_location(
                location_id=result.id,
                venue=result.venue,
                city=result.city,
                state=result.state,
            ),
        }

    def retrieve_by_slug(self, location_slug: str) -> dict[str, int | str]:
        """Retrieves location information.

        :param location_slug: Location slug string
        :return: A dictionary containing location ID, venue, city, state
            and slug string
        """
        try:
            slug = location_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(self, location_id: int) -> dict[str, Any]:
        """Retrieves location information and recordings.

        :param location_id: Location ID
        :return: A dictionary containing location ID, venue name, city,
            state, slug string and a list of recordings
        """
        if not valid_int_id(location_id):
            return {}

        info = self.retrieve_by_id(location_id)
        if not info:
            return {}

        info["recordings"] = self.recordings.retrieve_recordings_by_id(location_id)

        return info

    def retrieve_details_by_slug(self, location_slug: str) -> dict[str, Any]:
        """Retrieves location information and recordings.

        :param location_slug: Location slug string
        :return: A dictionary containing location ID, venue name, city,
            state, slug string and a list of recordings
        """
        try:
            slug = location_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.host.HostUtility`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.host import HostUtility


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("host_id", [2])
def test_host_utility_convert_id_to_slug(host_id: int):
    """Testing for :py:meth:`wwdtm.host.HostUtility.convert_id_to_slug`.

    :param host_id: Host ID to test converting into host slug string
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    slug = utility.convert_id_to_slug(host_id)

    assert slug, f"Host slug for ID {host_id} was not found"
    assert isinstance(slug, str), f"Invalid value returned for ID {host_id}"


@pytest.mark.parametrize("host_id", [-1])
def test_host_utility_convert_invalid_id_to_slug(host_id: int):
    """Negative testing for :py:meth:`wwdtm.host.HostUtility.convert_id_to_slug`.

    :param host_id: Host ID to test failing to convert into host slug
        string
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    slug = utility.convert_id_to_slug(host_id)

    assert not slug, f"Host slug for ID {host_id} was found"


@pytest.mark.parametrize("host_slug", ["tom-hanks"])
def test_host_utility_convert_slug_to_id(host_slug: str):
    """Testing for :py:meth:`wwdtm.host.HostUtility.convert_slug_to_id`.

    :param host_slug: Host slug string to test converting into host ID
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    id_ = utility.convert_slug_to_id(host_slug)

    assert id_, f"Host ID for slug {host_slug} was not found"
    assert isinstance(id_, int), f"Invalid value returned for slug {host_slug}"


@pytest.mark.parametrize("host_slug", ["tom-hanx"])
def test_host_utility_convert_invalid_slug_to_id(host_slug: str):
    """Negative testing for :py:meth:`wwdtm.host.HostUtility.convert_slug_to_id`.

    :param host_slug: Host slug string to test failing to convert into
        host ID
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    result = utility.convert_slug_to_id(host_slug)

    assert not result, f"Host ID for slug {host_slug} found"


@pytest.mark.parametrize("host_id", [2])
def test_host_utility_id_exists(host_id: int):
    """Testing for :py:meth:`wwdtm.host.HostUtility.id_exists`.

    :param host_id: Host ID to test if a host exists
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(host_id)

    assert result, f"Host ID {host_id} does not exist"


@pytest.mark.parametrize("host_id", [-1])
def test_host_utility_id_not_exists(host_id: int):
    """Negative testing for :py:meth:`wwdtm.host.HostUtility.id_exists()`.

    :param host_id: Host ID to test if a host does not exist
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    result = utility.id_exists(host_id)

    assert not result, f"Host ID {host_id} exists"


@pytest.mark.parametrize("host_slug", ["tom-hanks"])
def test_host_utility_slug_exists(host_slug: str):
    """Testing for :py:meth:`wwdtm.host.HostUtility.slug_exists`.

    :param host_slug: Host slug string to test if a host exists
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    result = utility.slug_exists(host_slug)

    assert result, f"Host slug {host_slug} does not exist"


@pytest.mark.parametrize("host_slug", ["tom-hanx"])
def test_host_utility_slug_not_exists(host_slug: str):
    """Negative testing for :py:meth:`wwdtm.host.HostUtility.slug_exists`.

    :param host_slug: Host slug string to test if a host does not exist
    """
    utility = HostUtility(connect_dict=get_connect_dict())
    result = utility.slug_exists(host_slug)

    assert not result, f"Host slug {host_slug} exists"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Show Data Utility Functions."""
import datetime
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.validation import valid_int_id


class ShowUtility:
    """Show information and utility class.

    Contains methods to convert between show ID and date, and to check
    if show IDs and dates exist.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

    def convert_date_to_id(self, year: int, month: int, day: int) -> int | None:
        """Converts a show date to the corresponding show ID.

        :param year: Year portion of a show date
        :param month: Month portion of a show date
        :param day: Day portion of a show date
        :return: Show ID if a corresponding value is found. Otherwise,
            ``None`` is returned
        """
        try:
            show_date = datetime.datetime(year, month, day)
        except ValueError:
            return None

        query = """
            SELECT showid FROM ww_shows WHERE showdate = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_date.isoformat(),))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def convert_id_to_date(self, show_id: int) -> str | None:
        """Converts a show ID to the corresponding show date.

        :param show_id: Show ID
        :return: Show date if a corresponding value is found. Otherwise,
            ``None`` is returned
        """
        if not valid_int_id(show_id):
            return None

        query = """
            SELECT showdate FROM ww_shows WHERE showid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_id,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0].isoformat()

        return None

    def date_exists(self, year: int, month: int, day: int) -> bool:
        """Validates if a show date exists.

        :param year: Year portion of a show date
        :param month: Month portion of a show date
        :param day: Day portion of a show date
        :return: ``True`` if the show date exists, otherwise ``False``
        """
        try:
            show_date = datetime.datetime(year, month, day)
        except ValueError:
            return False

        query = """
            SELECT showid FROM ww_shows WHERE showdate = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_date.isoformat(),))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)

    def id_exists(self, show_id: int) -> bool:
        """Validates if a show ID exists.

        :param show_id: Show ID
        :return: ``True`` if the show ID exists, otherwise ``False``
        """
        if not valid_int_id(show_id):
            return False

        query = """
            SELECT showid FROM ww_shows WHERE showid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_id,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Explicitly listing all modules in this package."""

from wwdtm import validation
from wwdtm.guest import Guest, GuestAppearances, GuestUtility
from wwdtm.host import Host, HostAppearances, HostUtility
from wwdtm.location import Location, LocationRecordings, LocationUtility
from wwdtm.panelist import (
    Panelist,
    PanelistAppearances,
    PanelistDecimalScores,
    PanelistScores,
    PanelistStatistics,
    PanelistUtility,
)
from wwdtm.scorekeeper import Scorekeeper, ScorekeeperAppearances, ScorekeeperUtility
from wwdtm.show import Show, ShowInfo, ShowInfoMultiple, ShowUtility

VERSION = "2.8.1"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Host Appearance Retrieval Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.host.utility import HostUtility
from wwdtm.validation import valid_int_id


class HostAppearances:
    """Host appearance information retrieval class.

    Contains methods used to retrieve host appearance information,
    including show flags.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = HostUtility(database_connection=self.database_connection)

    def retrieve_appearances_by_id(self, host_id: int) -> dict[str, Any]:
        """Retrieves host appearance information.

        :param host_id: Host ID
        :return: A dictionary containing host appearances with
            corresponding show dates and Best Of or Repeat show flags
        """
        if not valid_int_id(host_id):
            return {}

        query = """
            SELECT (
            SELECT COUNT(hm.showid) FROM ww_showhostmap hm
            JOIN ww_shows s ON s.showid = hm.showid
            WHERE s.bestof = 0 AND s.repeatshowid IS NULL AND
            hm.hostid = %s ) AS regular_shows, (
            SELECT COUNT(hm.showid) FROM ww_showhostmap hm
            JOIN ww_shows s ON s.showid = hm.showid
            WHERE hm.hostid = %s ) AS all_shows;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(
            query,
            (
                host_id,
                host_id,
            ),
        )
        result = cursor.fetchone()

        if result:
            appearance_counts = {
                "regular_shows": result.regular_shows,
                "all_shows": result.all_shows,
            }
        else:
            appearance_counts = {
                "regular_shows": 0,
                "all_shows": 0,
            }

        query = """
            SELECT hm.showid AS show_id, s.showdate AS date,
            s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
            hm.guest FROM ww_showhostmap hm
            JOIN ww_hosts h ON h.hostid = hm.hostid
            JOIN ww_shows s ON s.showid = hm.showid
            WHERE hm.hostid = %s
            ORDER BY s.showdate ASC;
            """
        cursor.execute(query, (host_id,))
        results = cursor.fetchall()
        cursor.close()

        if results:
            appearances = []
            for appearance in results:
                info = {
                    "show_id": appearance.show_id,
                    "date": appearance.date.isoformat(),
                    "best_of": bool(appearance.best_of),
                    "repeat_show": bool(appearance.repeat_show_id),
                    "guest": bool(appearance.guest),
                }
                appearances.append(info)

            return {
                "count": appearance_counts,
                "shows": appearances,
            }
        else:
            return {
                "count": appearance_counts,
                "shows": [],
            }

    def retrieve_appearances_by_slug(self, host_slug: str) -> dict[str, Any]:
        """Retrieves host appearance information.

        :param host_slug: Host slug string
        :return: A dictionary containing host appearances with
            corresponding show dates and Best Of or Repeat show flags
        """
        id_ = self.utility.convert_slug_to_id(host_slug)
        if not id_:
            return {}

        return self.retrieve_appearances_by_id(id_)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Location Data Utility Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.validation import valid_int_id


class LocationUtility:
    """Location information and utilities class.

    This class contains supporting functions used to check whether
    a location ID or slug string exists or to convert an ID to a slug
    string, or vice versa.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

    def convert_id_to_slug(self, location_id: int) -> str | None:
        """Converts a location ID to the corresponding location slug string.

        :param location_id: Location ID
        :return: Location slug string if a corresponding value is found.
            Otherwise, ``None`` is returned
        """
        if not valid_int_id(location_id):
            return None

        query = """
            SELECT locationslug FROM ww_locations WHERE locationid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (location_id,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def convert_slug_to_id(self, location_slug: str) -> int | None:
        """Converts a location slug string to the corresponding location ID.

        :param location_slug: Location slug string
        :return: Location ID if a corresponding value is found.
            Otherwise, ``None`` is returned
        """
        try:
            slug = location_slug.strip()
            if not slug:
                return None
        except ValueError:
            return None

        query = """
            SELECT locationid FROM ww_locations WHERE locationslug = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (slug,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def id_exists(self, location_id: int) -> bool:
        """Validates if a location ID exists.

        :param location_id: Location ID
        :return: ``True`` if the ID exists, otherwise ``False``
        """
        if not valid_int_id(location_id):
            return False

        query = """
            SELECT locationid FROM ww_locations WHERE locationid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (location_id,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)

    def slug_exists(self, location_slug: str) -> bool:
        """Validates if a location slug string exists.

        :param location_slug: Location slug string
        :return: ``True`` if the slug string exists, otherwise ``False``
        """
        try:
            slug = location_slug.strip()
            if not slug:
                return False
        except ValueError:
            return False

        query = """
            SELECT locationslug FROM ww_locations
            WHERE locationslug = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (slug,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)

    @staticmethod
    def slugify_location(
        location_id: int = None,
        venue: str = None,
        city: str = None,
        state: str = None,
    ) -> str:
        """Generates a slug string using the location ID, venue, city and/or state.

        :param location_id: Location ID
        :param venue: Location venue name
        :param city: City where the location venue is located
        :param state: State where the location venue is located
        :return: Location slug string
        :raises: ValueError
        """
        if venue and city and state:
            return slugify(f"{venue} {city} {state}")
        elif venue and city and not state:
            return slugify(f"{venue} {city}")
        elif location_id and venue and (not city and not state):
            return slugify(f"{id}-{venue}")
        elif location_id and city and state and not venue:
            return slugify(f"{id} {city} {state}")
        elif location_id:
            return f"location-{location_id}"
        else:
            raise ValueError("Invalid location information provided")



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Host Data Utility Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.validation import valid_int_id


class HostUtility:
    """Host information and utilities class.

    Contains methods used to convert between host ID and slug strings,
    and to check if host IDs and slug strings exist.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

    def convert_id_to_slug(self, host_id: int) -> str | None:
        """Converts a host ID to the corresponding host slug string.

        :param host_id: Host ID
        :return: Host slug string if a corresponding value is found.
            Otherwise, ``None`` is returned
        """
        if not valid_int_id(host_id):
            return None

        query = """
            SELECT hostslug FROM ww_hosts WHERE hostid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (host_id,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def convert_slug_to_id(self, host_slug: str) -> int | None:
        """Converts a host slug string to the corresponding host ID.

        :param host_slug: Host slug string
        :return: Host ID as an integer if a corresponding value is
            found. Otherwise, ``None`` is returned
        """
        try:
            slug = host_slug.strip()
            if not slug:
                return None
        except ValueError:
            return None

        query = """
            SELECT hostid FROM ww_hosts WHERE hostslug = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (slug,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def id_exists(self, host_id: int) -> bool:
        """Validates if a host ID exists.

        :param host_id: Host ID
        :return: ``True`` if the ID exists, otherwise ``False``
        """
        if not valid_int_id(host_id):
            return False

        query = """
            SELECT hostid FROM ww_hosts WHERE hostid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (host_id,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)

    def slug_exists(self, host_slug: str) -> bool:
        """Validates if a host slug string exists.

        :param host_slug: Host slug string
        :return: ``True`` if the slug string exists, otherwise ``False``
        """
        try:
            slug = host_slug.strip()
            if not slug:
                return False
        except ValueError:
            return False

        query = """
            SELECT hostslug FROM ww_hosts WHERE hostslug = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (slug,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Scorekeeper Data Utility Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.validation import valid_int_id


class ScorekeeperUtility:
    """Scorekeeper information and utilities class.

    Contains methods used to convert between scorekeeper ID and slug
    strings, and to check if scorekeeper IDs and slug strings exist.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

    def convert_id_to_slug(self, scorekeeper_id: int) -> str | None:
        """Converts a scorekeeper ID to the corresponding scorekeeper slug string.

        :param scorekeeper_id: Scorekeeper ID
        :return: Scorekeeper slug string if a corresponding value is
            found. Otherwise, ``None`` is returned
        """
        if not valid_int_id(scorekeeper_id):
            return None

        query = """
            SELECT scorekeeperslug FROM ww_scorekeepers
            WHERE scorekeeperid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (scorekeeper_id,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def convert_slug_to_id(self, scorekeeper_slug: str) -> int | None:
        """Converts a scorekeeper slug string to the corresponding scorekeeper ID.

        :param scorekeeper_slug: Scorekeeper slug string
        :return: Scorekeeper ID as an integer if a corresponding value
            is found. Otherwise, ``None`` is returned
        """
        try:
            slug = scorekeeper_slug.strip()
            if not slug:
                return None
        except ValueError:
            return None

        query = """
            SELECT scorekeeperid FROM ww_scorekeepers
            WHERE scorekeeperslug = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (slug,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def id_exists(self, scorekeeper_id: int) -> bool:
        """Validates if a scorekeeper ID exists.

        :param scorekeeper_id: Scorekeeper ID
        :return: ``True`` if the ID exists, otherwise ``False``
        """
        if not valid_int_id(scorekeeper_id):
            return False

        query = """
            SELECT scorekeeperid FROM ww_scorekeepers
            WHERE scorekeeperid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (scorekeeper_id,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)

    def slug_exists(self, scorekeeper_slug: str) -> bool:
        """Validates if a scorekeeper slug string exists.

        :param scorekeeper_slug: Scorekeeper slug string
        :return: ``True`` if the slug string exists, otherwise ``False``
        """
        try:
            slug = scorekeeper_slug.strip()
            if not slug:
                return False
        except ValueError:
            return False

        query = """
            SELECT scorekeeperslug FROM ww_scorekeepers
            WHERE scorekeeperslug = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (slug,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Host Data Retrieval Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.host.appearances import HostAppearances
from wwdtm.host.utility import HostUtility
from wwdtm.validation import valid_int_id


class Host:
    """Host information retrieval class.

    Contains methods used to retrieve host information, including IDs,
    names, slug strings and appearances.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.appearances = HostAppearances(database_connection=self.database_connection)
        self.utility = HostUtility(database_connection=self.database_connection)

    def retrieve_all(self) -> list[dict[str, int | str]]:
        """Retrieves host information for all hosts.

        :return: A list of dictionaries containing host ID, name, slug
            string and gender for each host
        """
        query = """
            SELECT hostid AS id, host AS name, hostslug AS slug,
            hostgender AS gender
            FROM ww_hosts
            ORDER BY host ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        hosts = []
        for row in results:
            hosts.append(
                {
                    "id": row.id,
                    "name": row.name,
                    "slug": row.slug if row.slug else slugify(row.name),
                    "gender": row.gender,
                }
            )

        return hosts

    def retrieve_all_details(self) -> list[dict[str, Any]]:
        """Retrieves host information and appearances for all hosts.

        :return: A list of dictionaries containing host ID, name, slug
            string, gender and a list of appearances with show flags for
            each host
        """
        query = """
            SELECT hostid AS id, host AS name, hostslug AS slug,
            hostgender AS gender
            FROM ww_hosts
            ORDER BY host ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        hosts = []
        for row in results:
            hosts.append(
                {
                    "id": row.id,
                    "name": row.name,
                    "slug": row.slug if row.slug else slugify(row.name),
                    "gender": row.gender,
                    "appearances": self.appearances.retrieve_appearances_by_id(row.id),
                }
            )

        return hosts

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all host IDs, sorted by host name.

        :return: A list of host IDs as integers
        """
        cursor = self.database_connection.cursor(dictionary=False)
        query = "SELECT hostid FROM ww_hosts ORDER BY host ASC;"
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self) -> list[str]:
        """Retrieves all host slug strings, sorted by host name.

        :return: A list of host slug strings
        """
        query = """
            SELECT hostslug FROM ww_hosts ORDER BY host ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, host_id: int) -> dict[str, Any]:
        """Retrieves host information.

        :param host_id: Host ID
        :return: A dictionary containing host ID, name, gender and slug string
        """
        if not valid_int_id(host_id):
            return {}

        query = """
            SELECT hostid AS id, host AS name, hostslug AS slug,
            hostgender AS gender
            FROM ww_hosts
            WHERE hostid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (host_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "id": result.id,
            "name": result.name,
            "slug": result.slug if result.slug else slugify(result.name),
            "gender": result.gender,
        }

    def retrieve_by_slug(self, host_slug: str) -> dict[str, Any]:
        """Retrieves host information.

        :param host_slug: Host slug string
        :return: A dictionary containing host ID, name, gender and slug
            string
        """
        try:
            slug = host_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(self, host_id: int) -> dict[str, Any]:
        """Retrieves host information and appearances.

        :param host_id: Host ID
        :return: A dictionary containing host ID, name, gender, slug
            string and a list of appearances with show flags
        """
        if not valid_int_id(host_id):
            return {}

        info = self.retrieve_by_id(host_id)
        if not info:
            return {}

        info["appearances"] = self.appearances.retrieve_appearances_by_id(host_id)

        return info

    def retrieve_details_by_slug(self, host_slug: str) -> dict[str, Any]:
        """Retrieves host information and appearances.

        :param host_slug: Host slug string
        :return: A dictionary containing host ID, name, slug string and
            a list of appearances with show flags
        """
        try:
            slug = host_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.scorekeeper.Scorekeeper`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.scorekeeper import Scorekeeper


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


def test_scorekeeper_retrieve_all():
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_all`."""
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    scorekeepers = scorekeeper.retrieve_all()

    assert scorekeepers, "No scorekeepers could be retrieved"
    assert "id" in scorekeepers[0], "'id' was not returned for the first list item"


def test_scorekeeper_retrieve_all_details():
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_all_details`."""
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    scorekeepers = scorekeeper.retrieve_all_details()

    assert scorekeepers, "No scorekeepers could be retrieved"
    assert "id" in scorekeepers[0], "'id' was not returned for first list item"
    assert (
        "appearances" in scorekeepers[0]
    ), "'appearances' was not returned for the first list item"


def test_scorekeeper_retrieve_all_ids():
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_all_ids`."""
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    ids = scorekeeper.retrieve_all_ids()

    assert ids, "No scorekeeper IDs could be retrieved"


def test_scorekeeper_retrieve_all_slugs():
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_all_slugs`."""
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    slugs = scorekeeper.retrieve_all_slugs()

    assert slugs, "No scorekeeper slug strings could be retrieved"


@pytest.mark.parametrize("scorekeeper_id", [13])
def test_scorekeeper_retrieve_by_id(scorekeeper_id: int):
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_by_id`.

    :param scorekeeper_id: Scorekeeper ID to test retrieving scorekeeper
        information
    """
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    info = scorekeeper.retrieve_by_id(scorekeeper_id)

    assert info, f"Scorekeeper ID {scorekeeper_id} not found"
    assert "name" in info, f"'name' was not returned for ID {scorekeeper_id}"


@pytest.mark.parametrize("scorekeeper_id", [13])
def test_scorekeeper_retrieve_details_by_id(scorekeeper_id: int):
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_details_by_id`.

    :param scorekeeper_id: Scorekeeper ID to test retrieving scorekeeper
        details
    """
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    info = scorekeeper.retrieve_details_by_id(scorekeeper_id)

    assert info, f"Scorekeeper ID {scorekeeper_id} not found"
    assert "name" in info, f"'name' was not returned for ID {scorekeeper_id}"
    assert (
        "appearances" in info
    ), f"'appearances' was not returned for ID {scorekeeper_id}"


@pytest.mark.parametrize("scorekeeper_slug", ["chioke-i-anson"])
def test_scorekeeper_retrieve_by_slug(scorekeeper_slug: str):
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_by_slug`.

    :param scorekeeper_slug: Scorekeeper slug string to test retrieving
        scorekeeper information
    """
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    info = scorekeeper.retrieve_by_slug(scorekeeper_slug)

    assert info, f"Scorekeeper slug {scorekeeper_slug} not found"
    assert "name" in info, f"'name' was not returned for slug {scorekeeper_slug}"


@pytest.mark.parametrize("scorekeeper_slug", ["chioke-i-anson"])
def test_scorekeeper_retrieve_details_by_slug(scorekeeper_slug: str):
    """Testing for :py:meth:`wwdtm.scorekeeper.Scorekeeper.retrieve_details_by_slug`.

    :param scorekeeper_slug: Scorekeeper slug string to test retrieving
        scorekeeper details
    """
    scorekeeper = Scorekeeper(connect_dict=get_connect_dict())
    info = scorekeeper.retrieve_details_by_slug(scorekeeper_slug)

    assert info, f"Scorekeeper slug {scorekeeper_slug} not found"
    assert "name" in info, f"'name' was not returned for slug {scorekeeper_slug}"
    assert (
        "appearances" in info
    ), f"'appearances' was not returned for slug {scorekeeper_slug}"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Scorekeeper Data Retrieval Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.scorekeeper.appearances import ScorekeeperAppearances
from wwdtm.scorekeeper.utility import ScorekeeperUtility
from wwdtm.validation import valid_int_id


class Scorekeeper:
    """Scorekeeper information retrieval class.

    Contains methods used to retrieve scorekeeper information, including
    IDs, names, slug strings and appearances.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.appearances = ScorekeeperAppearances(
            database_connection=self.database_connection
        )
        self.utility = ScorekeeperUtility(database_connection=self.database_connection)

    def retrieve_all(self) -> list[dict[str, Any]]:
        """Retrieves scorekeeper information for all scorekeepers.

        :return: A list of dictionaries containing scorekeeper ID, name,
            gender and slug string for each scorekeeper
        """
        query = """
            SELECT scorekeeperid AS id, scorekeeper AS name,
            scorekeeperslug AS slug, scorekeepergender AS gender
            FROM ww_scorekeepers
            ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        scorekeepers = []
        for row in results:
            scorekeepers.append(
                {
                    "id": row.id,
                    "name": row.name,
                    "gender": row.gender,
                    "slug": row.slug if row.slug else slugify(row.name),
                }
            )

        return scorekeepers

    def retrieve_all_details(self) -> list[dict[str, Any]]:
        """Retrieves scorekeeper information and appearances for all scorekeepers.

        :return: A list of dictionaries containing scorekeeper ID, name,
            slug string, gender and a list of appearances with show
            flags for each scorekeeper
        """
        query = """
            SELECT scorekeeperid AS id, scorekeeper AS name,
            scorekeeperslug AS slug, scorekeepergender AS gender
            FROM ww_scorekeepers
            ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        scorekeepers = []
        for row in results:
            scorekeepers.append(
                {
                    "id": row.id,
                    "name": row.name,
                    "slug": row.slug if row.slug else slugify(row.name),
                    "gender": row.gender,
                    "appearances": self.appearances.retrieve_appearances_by_id(row.id),
                }
            )

        return scorekeepers

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all scorekeeper IDs, sorted by scorekeeper name.

        :return: A list of scorekeeper IDs as integers
        """
        query = """
            SELECT scorekeeperid FROM ww_scorekeepers ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self) -> list[str]:
        """Retrieves all scorekeeper slug strings, sorted by scorekeeper name.

        :return: A list of scorekeeper slug strings
        """
        query = """
            SELECT scorekeeperslug FROM ww_scorekeepers ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, scorekeeper_id: int) -> dict[str, Any]:
        """Retrieves scorekeeper information.

        :param scorekeeper_id: Scorekeeper ID
        :return: A dictionary containing host ID, name, slug string and
            gender
        """
        if not valid_int_id(scorekeeper_id):
            return {}

        query = """
            SELECT scorekeeperid AS id, scorekeeper AS name,
            scorekeeperslug AS slug, scorekeepergender AS gender
            FROM ww_scorekeepers
            WHERE scorekeeperid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (scorekeeper_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "id": result.id,
            "name": result.name,
            "slug": result.slug if result.slug else slugify(result.name),
            "gender": result.gender,
        }

    def retrieve_by_slug(self, scorekeeper_slug: str) -> dict[str, Any]:
        """Retrieves scorekeeper information.

        :param scorekeeper_slug: Scorekeeper slug string
        :return: A dictionary containing host ID, name, slug string and
            gender
        """
        try:
            slug = scorekeeper_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(self, scorekeeper_id: int) -> dict[str, Any]:
        """Retrieves scorekeeper information and appearances.

        :param scorekeeper_id: Scorekeeper ID
        :return: A dictionaries containing scorekeeper ID, name, slug
            string, gender and a list of appearances with show flags
        """
        if not valid_int_id(scorekeeper_id):
            return {}

        info = self.retrieve_by_id(scorekeeper_id)
        if not info:
            return {}

        info["appearances"] = self.appearances.retrieve_appearances_by_id(
            scorekeeper_id
        )

        return info

    def retrieve_details_by_slug(self, scorekeeper_slug: str) -> dict[str, Any]:
        """Retrieves scorekeeper information and appearances.

        :param scorekeeper_slug: Scorekeeper slug string
        :return: A dictionaries containing scorekeeper ID, name, slug
            string, gender and a list of appearances with show flags
        """
        try:
            slug = scorekeeper_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Guest Data Retrieval Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.guest.appearances import GuestAppearances
from wwdtm.guest.utility import GuestUtility
from wwdtm.validation import valid_int_id


class Guest:
    """Guest information retrieval class.

    Contains methods used to retrieve Not My Job guest information,
    including IDs, names, slug strings, appearances and scores.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.appearances = GuestAppearances(
            database_connection=self.database_connection
        )
        self.utility = GuestUtility(database_connection=self.database_connection)

    def retrieve_all(self) -> list[dict[str, int | str]]:
        """Retrieves guest information for all guests.

        :return: A list of dictionaries containing guest ID, name and
            slug string for each guest
        """
        query = """
            SELECT guestid AS id, guest AS name, guestslug AS slug
            FROM ww_guests
            WHERE guestslug != 'none'
            ORDER BY guest ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        guests = []
        for row in results:
            guests.append(
                {
                    "id": row.id,
                    "name": row.name,
                    "slug": row.slug if row.slug else slugify(row.name),
                }
            )

        return guests

    def retrieve_all_details(self) -> list[dict[str, Any]]:
        """Retrieves guest information and appearances for all guests.

        :return: A list of dictionaries containing guest ID, name, slug
            string, and a list of appearances with show flags, scores
            and scoring exceptions
        """
        query = """
            SELECT guestid AS id, guest AS name, guestslug AS slug
            FROM ww_guests
            WHERE guestslug != 'none'
            ORDER BY guest ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        guests = []
        for row in results:
            guests.append(
                {
                    "id": row.id,
                    "name": row.name,
                    "slug": row.slug if row.slug else slugify(row.name),
                    "appearances": self.appearances.retrieve_appearances_by_id(row.id),
                }
            )

        return guests

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all guest IDs, sorted by guest name.

        :return: A list of guest IDs as integers
        """
        query = """
            SELECT guestid FROM ww_guests
            WHERE guestslug != 'none'
            ORDER BY guest ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self) -> list[str]:
        """Retrieves all guest slug strings, sorted by guest name.

        :return: A list of guest slug strings
        """
        query = """
            SELECT guestslug FROM ww_guests
            WHERE guestslug != 'none'
            ORDER BY guest ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, guest_id: int) -> dict[str, int | str]:
        """Retrieves guest information.

        :param guest_id: Guest ID
        :return: A dictionary containing guest ID, name and slug string
        """
        if not valid_int_id(guest_id):
            return {}

        query = """
            SELECT guestid AS id, guest AS name, guestslug AS slug
            FROM ww_guests
            WHERE guestid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query, (guest_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "id": result.id,
            "name": result.name,
            "slug": result.slug if result.slug else slugify(result.name),
        }

    def retrieve_by_slug(self, guest_slug: str) -> dict[str, int | str]:
        """Retrieves guest information.

        :param guest_slug: Guest slug string
        :return: A dictionary containing guest ID, name and slug string
        """
        try:
            slug = guest_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(self, guest_id: int) -> dict[str, Any]:
        """Retrieves guest information and appearances.

        :param guest_id: Guest ID
        :return: A dictionary containing guest ID, name, slug string,
            list of appearances with show flags, scores and scoring
            exceptions
        """
        if not valid_int_id(guest_id):
            return {}

        info = self.retrieve_by_id(guest_id)
        if not info:
            return {}

        info["appearances"] = self.appearances.retrieve_appearances_by_id(guest_id)

        return info

    def retrieve_details_by_slug(self, guest_slug: str) -> dict[str, Any]:
        """Retrieves guest information and appearances.

        :param guest_slug: Guest slug string
        :return: A dictionary containing guest ID, name, slug string,
            list of appearances with show flags, scores and scoring
            exceptions
        """
        try:
            slug = guest_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_)



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object :py:class:`wwdtm.show.ShowInfo`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.show import ShowInfo


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("show_id", [319, 1162])
def test_show_info_retrieve_bluff_info_by_id(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfo.retrieve_bluff_info_by_id`.

    :param show_id: Show ID to test retrieving show Bluff the Listener
        information
    """
    info = ShowInfo(connect_dict=get_connect_dict())
    bluff = info.retrieve_bluff_info_by_id(show_id)

    assert (
        isinstance(bluff, list) and bluff
    ), f"Bluff the Listener information for the show ID {show_id} could not be retrieved"

    assert "chosen_panelist" in bluff[0], (
        "'chosen_panelist' was not returned with panelist information for show ID "
        f"{show_id}"
    )
    assert "correct_panelist" in bluff[0], (
        "'correct_panelist' was not returned with panelist information for show ID "
        f"{show_id}"
    )


@pytest.mark.parametrize("show_id", [319, 1162])
def test_show_info_retrieve_core_info_by_id(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfo.retrieve_core_info_by_id`.

    :param show_id: Show ID to test retrieving show core information
    """
    info = ShowInfo(connect_dict=get_connect_dict())
    show = info.retrieve_core_info_by_id(show_id)

    assert show, f"Core information for show ID {show_id} could not be retrieved"

    assert (
        "id" in show
    ), f"'id' was not returned with core information for show ID {show_id}"
    assert (
        "description" in show
    ), f"'description' was not returned with show information for show ID {show_id}"


@pytest.mark.parametrize("show_id", [1162])
def test_show_info_retrieve_guest_info_by_id(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfo.retrieve_guest_info_by_id`.

    :param show_id: Show ID to test retrieving show guest information
    """
    info = ShowInfo(connect_dict=get_connect_dict())
    guests = info.retrieve_guest_info_by_id(show_id)

    assert guests, f"Guest information for show ID {show_id} could not be retrieved"
    assert (
        "id" in guests[0]
    ), f"'id' was not returned for the first guest for show ID {show_id}"
    assert (
        "score" in guests[0]
    ), f"'score' was not returned for the first guest for show ID {show_id}"


@pytest.mark.parametrize(
    "show_id, include_decimal_scores", [(1162, True), (1162, False)]
)
def test_show_info_retrieve_panelist_info_by_id(
    show_id: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.ShowInfo.retrieve_panelist_info_by_id`.

    :param show_id: Show ID to test retrieving show panelist information
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    info = ShowInfo(connect_dict=get_connect_dict())
    panelists = info.retrieve_panelist_info_by_id(
        show_id, include_decimal_scores=include_decimal_scores
    )

    assert (
        panelists
    ), f"Panelist information for show ID {show_id} could not be retrieved"
    assert (
        "id" in panelists[0]
    ), f"'id' was not returned for the first panelist for show ID {show_id}"
    assert (
        "score" in panelists[0]
    ), f"'score' was not returned for the first panelist for show ID {show_id}"
    if include_decimal_scores:
        assert (
            "score_decimal" in panelists[0]
        ), f"'score_decimal' was not returned for the first panelist for show ID {show_id}"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object :py:class:`wwdtm.show.Show`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.show import Show


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


def test_show_retrieve_all():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all()

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize("include_decimal_scores", [True, False])
def test_show_retrieve_all_details(include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_details`.

    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_all_details(include_decimal_scores=include_decimal_scores)

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


def test_show_retrieve_all_ids():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_ids`."""
    show = Show(connect_dict=get_connect_dict())
    ids = show.retrieve_all_ids()

    assert ids, "No show IDs could be retrieved"


def test_show_retrieve_all_dates():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_dates`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_dates()

    assert dates, "No show dates could be retrieved"


def test_show_retrieve_all_dates_tuple():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_dates_tuple`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_dates_tuple()

    assert dates, "No show dates could be retrieved"
    assert isinstance(dates[0], tuple), "First list item is not a tuple"


def test_show_retrieve_all_show_years_months():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_show_years_months`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_show_years_months()

    assert dates, "No dates could be retrieved"
    assert isinstance(dates[0], str), "First list item is not a string"


def test_show_retrieve_all_show_years_months_tuple():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_all_shows_years_months_tuple`."""
    show = Show(connect_dict=get_connect_dict())
    dates = show.retrieve_all_shows_years_months_tuple()

    assert dates, "No dates could be retrieved"
    assert isinstance(dates[0], tuple), "First list item is not a tuple"


@pytest.mark.parametrize("year, month, day", [(2020, 4, 25)])
def test_show_retrieve_by_date(year: int, month: int, day: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_date`.

    :param year: Four digit year to test retrieving a show's information
    :param month: One or two digit month to test retrieving a show's
        information
    :param day: One or two digit day to test retrieving a show's
        information
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_by_date(year, month, day)

    assert info, f"Show for date {year:04d}-{month:02d}-{day:02d} not found"
    assert (
        "date" in info
    ), f"'date' was not returned for show {year:04d}-{month:02d}-{day:02d}"


@pytest.mark.parametrize("date", ["2018-10-27"])
def test_show_retrieve_by_date_string(date: str):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_date_string`.

    :param date: Show date string in ``YYYY-MM-DD`` format to test
        retrieving a show's information
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_by_date_string(date)

    assert info, f"Show for date {date} not found"
    assert "date" in info, f"'date' was not returned for show {date}"


@pytest.mark.parametrize("show_id", [1162])
def test_show_retrieve_by_id(show_id: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_id`.

    :param show_id: Show ID to test retrieving show information
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_by_id(show_id)

    assert info, f"Show ID {show_id} not found"
    assert "date" in info, f"'date' was not returned for ID {show_id}"


@pytest.mark.parametrize("month, day", [(10, 28), (8, 19)])
def test_show_retrieve_by_month_day(month: int, day: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_month_day`.

    :param month: One or two digit month to test retrieving show details
    :param day: One or two digit day to test retrieving show details
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_by_month_day(month, day)

    assert shows, f"No shows could be retrieved for month {month:02d} and day {day:02d}"
    assert "id" in shows[0], (
        f"'id' was not returned for the first list item "
        f"for month {month:02d} and day {day:02d}"
    )


@pytest.mark.parametrize("year", [2018])
def test_show_retrieve_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_year`.

    :param year: Four digit year to test retrieving show information
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_by_year(year)

    assert shows, f"No shows could be retrieved for year {year:04d}"
    assert (
        "id" in shows[0]
    ), f"'id' was not returned for the first list item for year {year:04d}"


@pytest.mark.parametrize("year, month", [(1998, 1), (2018, 10)])
def test_show_retrieve_by_year_month(year: int, month: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_by_year_month`.

    :param year: Four digit year to test retrieving show information
    :param month: One or two digit month to test retrieving show
        information
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_by_year_month(year, month)

    assert shows, f"No shows could be retrieved for year/month {year:04d}-{month:02d}"
    assert "id" in shows[0], (
        f"'id' was not returned for the first list "
        f"item for year/month {year:02d}-{month:04d}"
    )


@pytest.mark.parametrize(
    "year, month, day, include_decimal_scores",
    [(2020, 4, 25, True), (2020, 4, 25, False)],
)
def test_show_retrieve_details_by_date(
    year: int, month: int, day: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_date`.

    :param year: Four digit year to test retrieving show details
    :param month: One or two digit month to test retrieving show details
    :param day: One or two digit day to test retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_date(
        year, month, day, include_decimal_scores=include_decimal_scores
    )

    assert info, f"Show for date {year:04d}-{month:02d}-{day:02d} not found"
    assert (
        "date" in info
    ), f"'date' was not returned for show {year:04d}-{month:02d}-{day:02d}"
    assert (
        "host" in info
    ), f"'host' was not returned for show {year:04d}-{month:02d}-{day:02d}"


@pytest.mark.parametrize(
    "date, include_decimal_scores", [("2018-10-27", True), ("2018-10-27", False)]
)
def test_show_retrieve_details_by_date_string(date: str, include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_date_string`.

    :param date: Show date string in ``YYYY-MM-DD`` format to test
        retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_date_string(
        date, include_decimal_scores=include_decimal_scores
    )

    assert info, f"Show for date {date} not found"
    assert "date" in info, f"'date' was not returned for show {date}"
    assert "host" in info, f"'host' was not returned for show {date}"


@pytest.mark.parametrize("date", ["1999-02-13", "2018-10-27"])
def test_show_retrieve_details_by_date_string_decimal(date: str):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_date_string` with decimal scores.

    :param date: Show date string in ``YYYY-MM-DD`` format to test
        retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_date_string(date, include_decimal_scores=True)

    assert info, f"Show for date {date} not found"
    assert "date" in info, f"'date' was not returned for show {date}"
    assert "host" in info, f"'host' was not returned for show {date}"


@pytest.mark.parametrize(
    "show_id, include_decimal_scores",
    [(1162, True), (1162, False), (1246, True), (1246, False)],
)
def test_show_retrieve_details_by_id(show_id: int, include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_id`.

    :param show_id: Show ID to test retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_id(
        show_id, include_decimal_scores=include_decimal_scores
    )

    assert info, f"Show ID {show_id} not found"
    assert "date" in info, f"'date' was not returned for ID {show_id}"
    assert "host" in info, f"'host' was not returned for ID {show_id}"


@pytest.mark.parametrize(
    "month, day, include_decimal_scores",
    [(10, 28, True), (10, 28, False), (8, 19, True), (8, 19, False)],
)
def test_show_retrieve_details_by_month_day(
    month: int, day: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_month_day`.

    :param month: One or two digit month to test retrieving show details
    :param day: One or two digit day to test retrieving show details
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_details_by_month_day(
        month, day, include_decimal_scores=include_decimal_scores
    )

    assert shows, f"No shows could be retrieved for month {month:02d} and day {day:02d}"
    assert "id" in shows[0], (
        f"'id' was not returned for the first list item "
        f"for month {month:02d} and day {day:02d}"
    )


@pytest.mark.parametrize("year, include_decimal_scores", [(2021, True), (2021, False)])
def test_show_retrieve_details_by_year(year: int, include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_year`.

    :param year: Four digit year to test retrieving show details
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_year(
        year, include_decimal_scores=include_decimal_scores
    )

    assert info, f"No shows could be retrieved for year {year:04d}"
    assert (
        "date" in info[0]
    ), f"'date' was not returned for first list item for year {year:04d}"
    assert (
        "host" in info[0]
    ), f"'host' was not returned for first list item for year {year:04d}"


@pytest.mark.parametrize(
    "year, month, include_decimal_scores", [(2020, 4, True), (2020, 4, False)]
)
def test_show_retrieve_details_by_year_month(
    year: int, month: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_details_by_year_month`.

    :param year: Four digit year to test retrieving show details
    :param month: One or two digit year to test retrieving show details
    """
    show = Show(connect_dict=get_connect_dict())
    info = show.retrieve_details_by_year_month(
        year, month, include_decimal_scores=include_decimal_scores
    )

    assert info, f"No shows could be retrieved for year/month {year:04d}-{month:02d}"
    assert "date" in info[0], (
        f"'date' was not returned for first list item "
        f"for year/month {year:04d}-{month:02d}"
    )
    assert "host" in info[0], (
        f"'host' was not returned for first list item "
        f"for year/month {year:04d}-{month:02d}"
    )


@pytest.mark.parametrize("year", [2018])
def test_show_retrieve_months_by_year(year: int):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_months_by_year`.

    :param year: Four digit year to test retrieving a list of months
    """
    show = Show(connect_dict=get_connect_dict())
    months = show.retrieve_months_by_year(year)

    assert months, f"No months could be retrieved for year {year:04d}"


def test_show_retrieve_recent():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_recent`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_recent()

    assert shows, "No shows could be retrieved"
    assert "id" in shows[0], "No Show ID returned for the first list item"


@pytest.mark.parametrize("include_decimal_scores", [True, False])
def test_show_retrieve_recent_details(include_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_recent_details`."""
    show = Show(connect_dict=get_connect_dict())
    shows = show.retrieve_recent_details(include_decimal_scores=include_decimal_scores)

    assert shows, "No shows could be retrieved"
    assert "date" in shows[0], "'date' was not returned for the first list item"
    assert "host" in shows[0], "'host' was not returned for first list item"


@pytest.mark.parametrize("year, use_decimal_scores", [(2018, True), (2018, False)])
def test_show_retrieve_scores_by_year(year: int, use_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_scores_by_year`.

    :param year: Four digit year to test retrieving scores for a show
        year
    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    show = Show(connect_dict=get_connect_dict())
    scores = show.retrieve_scores_by_year(year, use_decimal_scores=use_decimal_scores)

    assert scores, f"No scores could be retrieved by year {year:04d}"
    assert isinstance(scores[0], tuple), "First list item is not a tuple"


def test_show_retrieve_years():
    """Testing for :py:meth:`wwdtm.show.Show.retrieve_years`."""
    show = Show(connect_dict=get_connect_dict())
    years = show.retrieve_years()

    assert years, "No years could be retrieved"
    assert isinstance(years[0], int), "First list item is not a number"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Panelist Statistics Retrieval Functions."""
from decimal import Decimal
from typing import Any

import numpy
from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.panelist.decimal_scores import PanelistDecimalScores
from wwdtm.panelist.scores import PanelistScores
from wwdtm.panelist.utility import PanelistUtility
from wwdtm.validation import valid_int_id


class PanelistStatistics:
    """Panelist statistics information retrieval and calculation class.

    Contains methods used to retrieve Bluff the Listener, ranking
    and scoring statistics.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.scores = PanelistScores(database_connection=self.database_connection)
        self.scores_decimal = PanelistDecimalScores(
            database_connection=self.database_connection
        )
        self.utility = PanelistUtility(database_connection=self.database_connection)

    def retrieve_bluffs_by_id(self, panelist_id: int) -> dict[str, int]:
        """Retrieve panelist Bluff the Listener statistics.

        :param panelist_id: Panelist ID
        :return: A dictionary containing number of times a panelist's
            Bluff the Listener story was chosen and number of times
            they had the correct story
        """
        query = """
            SELECT (
            SELECT COUNT(blm.chosenbluffpnlid) FROM ww_showbluffmap blm
            JOIN ww_shows s ON s.showid = blm.showid
            WHERE s.repeatshowid IS NULL AND blm.chosenbluffpnlid = %s
            ) AS chosen, (
            SELECT COUNT(blm.correctbluffpnlid) FROM ww_showbluffmap blm
            JOIN ww_shows s ON s.showid = blm.showid
            WHERE s.repeatshowid IS NULL AND blm.correctbluffpnlid = %s
            ) AS correct;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(
            query,
            (
                panelist_id,
                panelist_id,
            ),
        )
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "chosen": result.chosen,
            "correct": result.correct,
        }

    def retrieve_bluffs_by_slug(self, panelist_slug: str) -> dict[str, int]:
        """Retrieve panelist Bluff the Listener statistics.

        :param panelist_slug: Panelist slug string
        :return: A dictionary containing number of times a panelist's
            Bluff the Listener story was chosen and number of times
            they had the correct story
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_bluffs_by_id(id_)

    def retrieve_rank_info_by_id(self, panelist_id: int) -> dict[str, int]:
        """Retrieves panelist ranking information.

        :param panelist_id: Panelist ID
        :return: A dictionary containing the number of times a panelist
            has finished in 1st, 1st tied, 2nd, 2nd tied and 3rd place
        """
        if not valid_int_id(panelist_id):
            return {}

        query = """
            SELECT (
            SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s AND pm.showpnlrank = '1' AND
            s.bestof = 0 and s.repeatshowid IS NULL) as 'first', (
            SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s AND pm.showpnlrank = '1t' AND
            s.bestof = 0 and s.repeatshowid IS NULL) as 'first_tied', (
            SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s AND pm.showpnlrank = '2' AND
            s.bestof = 0 and s.repeatshowid IS NULL) as 'second', (
            SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s AND pm.showpnlrank = '2t' AND
            s.bestof = 0 and s.repeatshowid IS NULL) as 'second_tied', (
            SELECT COUNT(pm.showpnlrank) FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s AND pm.showpnlrank = '3' AND
            s.bestof = 0 and s.repeatshowid IS NULL
            ) as 'third';
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(
            query,
            (
                panelist_id,
                panelist_id,
                panelist_id,
                panelist_id,
                panelist_id,
            ),
        )
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        return {
            "first": result.first,
            "first_tied": result.first_tied,
            "second": result.second,
            "second_tied": result.second_tied,
            "third": result.third,
        }

    def retrieve_rank_info_by_slug(self, panelist_slug: str) -> dict[str, int]:
        """Retrieves panelist ranking information.

        :param panelist_slug: Panelist slug string
        :return: A dictionary containing the number of times a panelist
            has finished in 1st, 1st tied, 2nd, 2nd tied and 3rd place
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_rank_info_by_id(id_)

    def retrieve_statistics_by_id(
        self, panelist_id: int, include_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves and calculates panelist statistics.

        :param panelist_id: Panelist ID
        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A dictionary containing panelist scoring and ranking
            statistics
        """
        if not valid_int_id(panelist_id):
            return {}

        score_data = self.scores.retrieve_scores_by_id(panelist_id)
        ranks = self.retrieve_rank_info_by_id(panelist_id)
        if not score_data or not ranks:
            return {}

        if include_decimal_scores:
            score_data_decimal = self.scores_decimal.retrieve_scores_by_id(panelist_id)
            if not score_data_decimal:
                return {}

        appearance_count = len(score_data)
        scoring = {
            "minimum": int(numpy.amin(score_data)),
            "maximum": int(numpy.amax(score_data)),
            "mean": round(numpy.mean(score_data), 5),
            "median": int(numpy.median(score_data)),
            "standard_deviation": round(numpy.std(score_data), 5),
            "total": int(numpy.sum(score_data)),
        }

        if include_decimal_scores:
            scoring_decimal = {
                "minimum": Decimal(numpy.amin(score_data_decimal)),
                "maximum": Decimal(numpy.amax(score_data_decimal)),
                "mean": round(Decimal(numpy.mean(score_data_decimal)), 5),
                "median": Decimal(numpy.median(score_data_decimal)),
                "standard_deviation": round(Decimal(numpy.std(score_data_decimal)), 5),
                "total": Decimal(numpy.sum(score_data_decimal)),
            }

        ranks_first = round(100 * (ranks["first"] / appearance_count), 5)
        ranks_first_tied = round(100 * (ranks["first_tied"] / appearance_count), 5)
        ranks_second = round(100 * (ranks["second"] / appearance_count), 5)
        ranks_second_tied = round(100 * (ranks["second_tied"] / appearance_count), 5)
        ranks_third = round(100 * (ranks["third"] / appearance_count), 5)

        ranks_percentage = {
            "first": ranks_first,
            "first_tied": ranks_first_tied,
            "second": ranks_second,
            "second_tied": ranks_second_tied,
            "third": ranks_third,
        }

        ranking = {
            "rank": ranks,
            "percentage": ranks_percentage,
        }

        if include_decimal_scores:
            return {
                "scoring": scoring,
                "scoring_decimal": scoring_decimal,
                "ranking": ranking,
            }
        else:
            return {
                "scoring": scoring,
                "ranking": ranking,
            }

    def retrieve_statistics_by_slug(
        self, panelist_slug: str, include_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves and calculates panelist statistics.

        :param panelist_slug: Panelist slug string
        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A dictionary containing panelist scoring and ranking
            statistics
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_statistics_by_id(
            id_, include_decimal_scores=include_decimal_scores
        )



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.location.LocationRecordings`."""
import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.location import LocationRecordings


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("location_id", [95])
def test_location_recordings_retrieve_recordings_by_id(location_id: int):
    """Testing for :py:meth:`wwdtm.location.LocationRecordings.retrieve_recordings_by_id`.

    :param location_id: Location ID to test retrieving location
        recordings
    """
    recordings = LocationRecordings(connect_dict=get_connect_dict())
    recording = recordings.retrieve_recordings_by_id(location_id)

    assert "count" in recording, f"'count' was not returned for ID {location_id}"
    assert "shows" in recording, f"'shows' was not returned for ID {location_id}"


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-il"])
def test_location_recordings_retrieve_recordings_by_slug(location_slug: str):
    """Testing for :py:meth:`wwdtm.location.LocationRecordings.retrieve_recordings_by_slug`.

    :param location_slug: Location slug string to test retrieving
        location recordings
    """
    recordings = LocationRecordings(connect_dict=get_connect_dict())
    recording = recordings.retrieve_recordings_by_slug(location_slug)

    assert "count" in recording, f"'count' was not returned for slug {location_slug}"
    assert "shows" in recording, f"'shows' was not returned for slug {location_slug}"



# Copyright (c) 2018-2024 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Don't Tell Me! Stats Panelist Appearance Retrieval Functions."""
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.panelist.utility import PanelistUtility
from wwdtm.validation import valid_int_id


class PanelistAppearances:
    """Panelist appearance information retrieval class.

    Contains methods used to retrieve panelist appearance information
    and statistics, scores and rankings.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = PanelistUtility(database_connection=self.database_connection)

    def retrieve_appearances_by_id(
        self, panelist_id: int, use_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves panelist appearance information.

        :param panelist_id: Panelist ID
        :param use_decimal_scores: A boolean to determine if decimal
            scores returned instead of integer scores
        :return: A dictionary containing appearance statistics, scoring
            and ranking details
        """
        if not valid_int_id(panelist_id):
            return {}

        query = """
            SELECT (
            SELECT COUNT(pm.showid) FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE s.bestof = 0 AND s.repeatshowid IS NULL AND
            pm.panelistid = %s ) AS regular_shows, (
            SELECT COUNT(pm.showid) FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE pm.panelistid = %s ) AS all_shows, (
            SELECT COUNT(pm.panelistid) FROM ww_showpnlmap pm
            JOIN ww_shows s ON pm.showid = s.showid
            WHERE pm.panelistid = %s AND s.bestof = 0 AND
            s.repeatshowid IS NULL
            AND pm.panelistscore IS NOT NULL )
            AS shows_with_scores;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(
            query,
            (
                panelist_id,
                panelist_id,
                panelist_id,
            ),
        )
        result = cursor.fetchone()

        if result:
            appearance_counts = {
                "regular_shows": result.regular_shows,
                "all_shows": result.all_shows,
                "shows_with_scores": result.shows_with_scores,
            }
        else:
            appearance_counts = {
                "regular_shows": 0,
                "all_shows": 0,
                "shows_with_scores": 0,
            }

        query = """
            SELECT MIN(s.showid) AS first_id, MIN(s.showdate) AS first,
            MAX(s.showid) AS most_recent_id, MAX(s.showdate) AS most_recent
            FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            WHERE s.bestof = 0 AND s.repeatshowid IS NULL
            AND pm.panelistid = %s
            ORDER BY s.showdate ASC;
            """
        cursor.execute(query, (panelist_id,))
        result = cursor.fetchone()

        if result and result.first_id:
            first = {
                "show_id": result.first_id,
                "show_date": result.first.isoformat(),
            }

            most_recent = {
                "show_id": result.most_recent_id,
                "show_date": result.most_recent.isoformat(),
            }

            milestones = {
                "first": first,
                "most_recent": most_recent,
            }

            appearance_info = {
                "milestones": milestones,
            }
        else:
            appearance_info = {
                "milestones": None,
            }

        if use_decimal_scores:
            query = """
                SELECT pm.showid AS show_id, s.showdate AS date,
                s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
                pm.panelistlrndstart AS start,
                pm.panelistlrndstart_decimal AS start_decimal,
                pm.panelistlrndcorrect AS correct,
                pm.panelistlrndcorrect_decimal AS correct_decimal,
                pm.panelistscore AS score,
                pm.panelistscore_decimal AS score_decimal,
                pm.showpnlrank AS pnl_rank FROM ww_showpnlmap pm
                JOIN ww_panelists p ON p.panelistid = pm.panelistid
                JOIN ww_shows s ON s.showid = pm.showid
                WHERE pm.panelistid = %s
                ORDER BY s.showdate ASC;
                """
        else:
            query = """
                SELECT pm.showid AS show_id, s.showdate AS date,
                s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
                pm.panelistlrndstart AS start,
                pm.panelistlrndcorrect AS correct,
                pm.panelistscore AS score,
                pm.showpnlrank AS pnl_rank FROM ww_showpnlmap pm
                JOIN ww_panelists p ON p.panelistid = pm.panelistid
                JOIN ww_shows s ON s.showid = pm.showid
                WHERE pm.panelistid = %s
                ORDER BY s.showdate ASC;
                """

        cursor.execute(query, (panelist_id,))
        results = cursor.fetchall()
        cursor.close()

        if result:
            appearances = []
            for appearance in results:
                info = {
                    "show_id": appearance.show_id,
                    "date": appearance.date.isoformat(),
                    "best_of": bool(appearance.best_of),
                    "repeat_show": bool(appearance.repeat_show_id),
                    "lightning_round_start": appearance.start,
                    "lightning_round_start_decimal": appearance.start_decimal
                    if "start_decimal" in appearance._fields
                    else None,
                    "lightning_round_correct": appearance.correct,
                    "lightning_round_correct_decimal": appearance.correct_decimal
                    if "correct_decimal" in appearance._fields
                    else None,
                    "score": appearance.score,
                    "score_decimal": appearance.score_decimal
                    if "score_decimal" in appearance._fields
                    else None,
                    "rank": appearance.pnl_rank
                    if appearance.pnl_rank is not None
                    else None,
                }
                appearances.append(info)

            appearance_info["count"] = appearance_counts
            appearance_info["shows"] = appearances
        else:
            appearance_info["count"] = appearance_counts
            appearance_info["shows"] = []

        return appearance_info

    def retrieve_appearances_by_slug(
        self, panelist_slug: str, use_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves panelist appearance information.

        :param panelist_slug: Panelist slug string
        :param use_decimal_scores: A boolean to determine if decimal
            scores returned instead of integer scores
        :return: A dictionary containing appearance statistics, scoring
            and ranking details
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_appearances_by_id(
            id_, use_decimal_scores=use_decimal_scores
        )

    def retrieve_yearly_appearances_by_id(self, panelist_id: int) -> dict[int, int]:
        """Retrieves panelist appearance counts grouped by year.

        :param panelist_id: Panelist ID
        :return: A dictionary containing years as keys and appearance
            counts for each year as values
        """
        if not valid_int_id(panelist_id):
            return {}

        query = """
            SELECT DISTINCT YEAR(s.showdate) AS year
            FROM ww_shows s
            ORDER BY YEAR(s.showdate) ASC;
            """
        cursor = self.database_connection.cursor(named_tuple=True)
        cursor.execute(query)
        results = cursor.fetchall()

        if not results:
            return {}

        years = {}
        for row in results:
            years[row.year] = 0

        query = """
            SELECT YEAR(s.showdate) AS year,
            COUNT(p.panelist) AS count
            FROM ww_showpnlmap pm
            JOIN ww_shows s ON s.showid = pm.showid
            JOIN ww_panelists p ON p.panelistid = pm.panelistid
            WHERE pm.panelistid = %s AND s.bestof = 0
            AND s.repeatshowid IS NULL
            GROUP BY p.panelist, YEAR(s.showdate)
            ORDER BY p.panelist ASC, YEAR(s.showdate) ASC;
            """
        cursor.execute(query, (panelist_id,))
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        for row in results:
            years[row.year] = row.count

        return years

    def retrieve_yearly_appearances_by_slug(self, panelist_slug: str) -> dict[int, int]:
        """Retrieves panelist appearance counts grouped by year.

        :param panelist_slug: Panelist slug string
        :return: A dictionary containing years as keys and appearance
            counts for each year as values
        """
        id_ = self.utility.convert_slug_to_id(panelist_slug)
        if not id_:
            return {}

        return self.retrieve_yearly_appearances_by_id(id_)
